/*  Screem:  screem-pref_dialog.c
 *
 *  Copyright (C) 2003  David A Knight
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  For contact information with the author of this source code please see
 *  the AUTHORS file.  If there is no AUTHORS file present then check the
 *  about box under the help menu for a contact address
 */

#include <config.h>

#include "screem-pref-dialog.h"

#include <string.h>

#include <libgnome/gnome-i18n.h>

#include <libgnomeui/gnome-color-picker.h>
#include <libgnomeui/gnome-entry.h>
#include <libgnomeui/gnome-file-entry.h>
#include <libgnomeui/gnome-icon-entry.h>
#include <libgnomeui/gnome-font-picker.h>

#include <gtk/gtkhbox.h>
#include <gtk/gtkhseparator.h>
#include <gtk/gtkvbox.h>
#include <gtk/gtknotebook.h>
#include <gtk/gtkimage.h>
#include <gtk/gtklabel.h>
#include <gtk/gtkstock.h>

#include <gtk/gtkscrolledwindow.h>
#include <gtk/gtktreeview.h>
#include <gtk/gtktreeselection.h>
#include <gtk/gtkcellrenderertext.h>
#include <gtk/gtkcellrendererpixbuf.h>
#include <gtk/gtktreestore.h>

#include <gtk/gtktogglebutton.h>
#include <gtk/gtkspinbutton.h>
#include <gtk/gtkentry.h>
#include <gtk/gtkcombo.h>

#include <gconf/gconf-client.h>

struct ScreemPrefDialogPrivate {
	GtkNotebook *notebook;
	GtkTreeView *view;

	GtkImage *logo;
	
	GtkTreeStore *categories;

	GConfClient *client;
	GSList *notify_handles;
};

enum {
	CATEGORY_NAME,
	CATEGORY_ICON,
	CATEGORY_PAGE,
	CATEGORY_COLS
};

enum {
	PROP_0,
	PROP_LOGO
};

static void screem_pref_dialog_init( ScreemPrefDialog *pref_dialog );
static void screem_pref_dialog_class_init( ScreemPrefDialogClass *pref_dialog );
static void screem_pref_dialog_finalize( GObject *pref_dialog );
static void screem_pref_dialog_set_prop( GObject *object, guint prop_id,
					 const GValue *value, GParamSpec *spec );
static void screem_pref_dialog_get_prop( GObject *object, guint prop_id,
					 GValue *value, GParamSpec *spec );

#define PARENT_TYPE GTK_TYPE_DIALOG

static gpointer parent_class;


static void screem_pref_dialog_cat_select( GtkTreeSelection *selection,
					    ScreemPrefDialog *dialog );


static void screem_pref_dialog_class_init( ScreemPrefDialogClass *klass )
{
	GObjectClass *object_class;

	object_class = G_OBJECT_CLASS( klass );
	parent_class = g_type_class_peek_parent( klass );

	object_class->finalize = screem_pref_dialog_finalize;

	object_class->set_property = screem_pref_dialog_set_prop;
	object_class->get_property = screem_pref_dialog_get_prop;

	g_object_class_install_property(object_class,
					PROP_LOGO,
					g_param_spec_object("logo",
							    "Application Logo",
							    "The Application Logo",
							    GDK_TYPE_PIXBUF,
							    G_PARAM_READWRITE |
							    G_PARAM_CONSTRUCT)
					);

}

static void screem_pref_dialog_init( ScreemPrefDialog *pref_dialog )
{
	ScreemPrefDialogPrivate *priv;
	GtkDialog *dialog;
	GtkWidget *label;
	GtkBox *box;
	GtkTreeStore *store;
	GtkWidget *window;
	GtkTreeView *view;
	GtkTreeSelection *selection;
	GtkCellRenderer *prenderer;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;

	GtkWidget *cbox;
	
	dialog = GTK_DIALOG( pref_dialog );
	gtk_dialog_set_has_separator( dialog, FALSE );
	
	priv = pref_dialog->priv = g_new0( ScreemPrefDialogPrivate, 1 );
	
	priv->client = gconf_client_get_default();
	
	box = GTK_BOX( gtk_hbox_new( FALSE, 0 ) );
	gtk_box_set_spacing( box, 6 );
	gtk_box_pack_start( GTK_BOX( dialog->vbox ), GTK_WIDGET( box ), TRUE, TRUE, 0 );
	
	priv->notebook = GTK_NOTEBOOK( gtk_notebook_new() );
	gtk_notebook_set_show_tabs( priv->notebook, FALSE );
	gtk_notebook_set_show_border( priv->notebook, FALSE );
	
	label = gtk_label_new( "" );
	priv->logo = GTK_IMAGE( gtk_image_new() );
	gtk_notebook_append_page( priv->notebook, GTK_WIDGET( priv->logo ),
				label );
	
	store = gtk_tree_store_new( CATEGORY_COLS,
				    G_TYPE_STRING,
				    GDK_TYPE_PIXBUF,
				    G_TYPE_INT );
	priv->categories = store;

	window = gtk_scrolled_window_new( NULL, NULL );
	gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW( window ),
					GTK_POLICY_NEVER,
					GTK_POLICY_AUTOMATIC );
	gtk_scrolled_window_set_shadow_type( GTK_SCROLLED_WINDOW( window ),
						GTK_SHADOW_IN );
	view = GTK_TREE_VIEW( gtk_tree_view_new_with_model( GTK_TREE_MODEL( store ) ) );
	priv->view = view;
	gtk_container_add( GTK_CONTAINER( window ), GTK_WIDGET( view ) );
	cbox = gtk_vbox_new( FALSE, 0 );
	gtk_container_set_border_width( GTK_CONTAINER( cbox ), 6 );
	label = gtk_label_new_with_mnemonic( _( "C_ategories:" ) );
	gtk_label_set_mnemonic_widget( GTK_LABEL( label ), 
					GTK_WIDGET( view ) );
	gtk_misc_set_alignment( GTK_MISC( label ), 0.0, 0.0 );
	gtk_box_pack_start( GTK_BOX( cbox ), label, FALSE, FALSE, 6 );

	selection = gtk_tree_view_get_selection( view );
	g_signal_connect( G_OBJECT( selection ), "changed",
			  G_CALLBACK( screem_pref_dialog_cat_select ),
			  pref_dialog );
	
	prenderer = gtk_cell_renderer_pixbuf_new();
	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new();
	
	gtk_tree_view_column_pack_start( column, prenderer, FALSE );
	gtk_tree_view_column_pack_start( column, renderer, TRUE );
	gtk_tree_view_column_set_resizable( column, FALSE );
	gtk_tree_view_append_column( GTK_TREE_VIEW( view ), column );
	gtk_tree_view_column_set_attributes( column, renderer,
					     "text", 
					     CATEGORY_NAME,
					     NULL );
	gtk_tree_view_column_set_attributes( column, prenderer,
					     "pixbuf",
					     CATEGORY_ICON,
					     NULL );

	gtk_tree_view_set_headers_visible( GTK_TREE_VIEW( view ), FALSE );
	
	gtk_box_pack_start( GTK_BOX( cbox ), GTK_WIDGET( window ), TRUE, TRUE, 0 );
	gtk_box_pack_start( box, cbox, FALSE, TRUE, 0 );
	gtk_box_pack_start( box, GTK_WIDGET( priv->notebook ), TRUE, TRUE, 0 );
	
	gtk_dialog_add_button( dialog, GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE );

	gtk_widget_show_all( GTK_WIDGET( box ) );
}

static void screem_pref_dialog_finalize( GObject *pref_dialog )
{
	ScreemPrefDialog *dialog;
	ScreemPrefDialogPrivate *priv;
	GSList *list;
	
	dialog = SCREEM_PREF_DIALOG( pref_dialog );
	priv = dialog->priv;

	for( list = priv->notify_handles; list; list = list->next ) {
		gconf_client_notify_remove( priv->client,
					    GPOINTER_TO_UINT( list->data ) );
	}
	g_slist_free( priv->notify_handles );
	
	g_object_unref( priv->client );
	
	g_free( priv );
	
	G_OBJECT_CLASS( parent_class )->finalize( pref_dialog );
}

static void screem_pref_dialog_set_prop( GObject *object, guint prop_id,
					 const GValue *value, GParamSpec *spec )
{
	ScreemPrefDialog *dialog;
	ScreemPrefDialogPrivate *priv;
	GdkPixbuf *pixbuf;
	
	dialog = SCREEM_PREF_DIALOG( object );
	priv = dialog->priv;

	switch( prop_id ) {
		case PROP_LOGO:
			pixbuf = g_value_get_object( value );			
			gtk_image_set_from_pixbuf( priv->logo, pixbuf );
			break;
		default:
			break;
	};
}

static void screem_pref_dialog_get_prop( GObject *object, guint prop_id,
					 GValue *value, GParamSpec *spec )
{
	ScreemPrefDialog *dialog;
	ScreemPrefDialogPrivate *priv;
	GdkPixbuf *pixbuf;
	
	dialog = SCREEM_PREF_DIALOG( object );
	priv = dialog->priv;

	switch( prop_id ) {
		case PROP_LOGO:
			pixbuf = gtk_image_get_pixbuf( priv->logo );
			g_value_set_object( value, G_OBJECT( pixbuf ) );
			break;
		default:
			break;
	};
}

GType screem_pref_dialog_get_type()
{
	static GType type = 0;

	if( ! type ) {
		static const GTypeInfo info = {
			sizeof( ScreemPrefDialogClass ),
			NULL, /* base init */
			NULL, /* base finalise */
			(GClassInitFunc)screem_pref_dialog_class_init,
			NULL, /* class finalise */
			NULL, /* class data */
			sizeof( ScreemPrefDialog ),
			0, /* n_preallocs */
			(GInstanceInitFunc)screem_pref_dialog_init
		};

		type = g_type_register_static( PARENT_TYPE,
					       "ScreemPrefDialog",
					       &info, 0 );
	}

	return type;
}

/* static stuff */
static void screem_pref_dialog_cat_select( GtkTreeSelection *selection,
					    ScreemPrefDialog *dialog )
{
	GtkTreeModel *model;
	GtkTreeIter it;
	gint page;
	GtkNotebook *notebook;

	page = -1;
	notebook = dialog->priv->notebook;

	if( gtk_tree_selection_get_selected( selection, &model, &it ) ) {
		gtk_tree_model_get( model, &it, CATEGORY_PAGE, &page, -1 );
		if( page != -1 ) {
			/* valid page */
			gtk_notebook_set_current_page( notebook, page );
		} else {
			gtk_notebook_set_current_page( notebook, 0 );
		}
	}
}

gboolean screem_pref_dialog_find_category( ScreemPrefDialog *dialog,
					   GtkTreeIter *it,
					   const gchar *category )
{
	gboolean ret;
	GtkTreeModel *model;
	const gchar *name;
	
	g_return_val_if_fail( SCREEM_IS_PREF_DIALOG( dialog ), FALSE );
	g_return_val_if_fail( it != NULL, FALSE );
	g_return_val_if_fail( category != NULL, FALSE );
	
	ret = FALSE;

	model = GTK_TREE_MODEL( dialog->priv->categories );

	do {
		gtk_tree_model_get( model, it, CATEGORY_NAME, &name, -1 );
		ret = ( ! strcmp( category, name ) );
	} while( ( ! ret ) && gtk_tree_model_iter_next( model, it ) );
		
	return ret;
}

static void screem_pref_dialog_add_category( ScreemPrefDialog *dialog,
						const gchar *category,
						const gchar *stock_id,
						GtkTreeIter *parent,
						GtkTreeIter *it,
						gint page )
{
	GtkTreeStore *store;
	GdkPixbuf *pixbuf;
	
	g_return_if_fail( SCREEM_IS_PREF_DIALOG( dialog ) );
	g_return_if_fail( category != NULL );
	g_return_if_fail( it != NULL );

	store = dialog->priv->categories;
	
	gtk_tree_store_append( store, it, parent );
	gtk_tree_store_set( store, it, 
			    CATEGORY_NAME, category,
			    CATEGORY_PAGE, page,
			    -1 );

	if( stock_id ) {
		pixbuf = gtk_widget_render_icon( GTK_WIDGET( dialog ),
						 stock_id,
						 GTK_ICON_SIZE_MENU,
						 "" );
		if( pixbuf ) {
			gtk_tree_store_set( store, it,
					    CATEGORY_ICON, pixbuf,
					    -1 );
		} else {
			g_warning( "Unable to render pixbuf for %s\n", 
				   stock_id );
		}
	}
}

GtkWidget *screem_pref_dialog_new_page( ScreemPrefDialog *dialog, 
					const gchar *name, 
					const gchar *stock_id,
					GtkWidget *child )
{
	GtkWidget *box;

	GtkWidget *header;
	GtkWidget *image;
	GtkLabel *label;
	gchar *escaped;
	gchar *markup;
	
	GtkWidget *sep;
	
	box = gtk_vbox_new( FALSE, 0 );
	gtk_container_set_border_width( GTK_CONTAINER( box ), 12 );
	gtk_box_set_spacing( GTK_BOX( box ), 6 );

	header = gtk_hbox_new( FALSE, 0 );
	gtk_box_set_spacing( GTK_BOX( header ), 6 );
	gtk_box_pack_start( GTK_BOX( box ), header, FALSE, FALSE, 0 );

	/* icon / label */
	if( stock_id ) {
		image = gtk_image_new_from_stock( stock_id, GTK_ICON_SIZE_BUTTON );
		gtk_box_pack_start( GTK_BOX( header ), image, FALSE, FALSE, 0 );
	}
	escaped = g_markup_escape_text( name, strlen( name ) );
	markup = g_strdup_printf( "<span weight=\"bold\" size=\"x-large\">%s</span>",
				   escaped );
	g_free( escaped );
	label = GTK_LABEL( gtk_label_new( markup ) );
	gtk_label_set_use_markup( label, TRUE );
	g_free( markup );
	gtk_box_pack_start( GTK_BOX( header ), GTK_WIDGET( label ), 
				FALSE, TRUE, 0 );

	/* separator */
	sep = gtk_hseparator_new();
	gtk_box_pack_start( GTK_BOX( box ), sep, FALSE, TRUE, 0 );
	
	/* child widget */
	gtk_box_pack_start( GTK_BOX( box ), child, TRUE, TRUE, 0 );

	gtk_widget_show_all( box );
	
	return box;
}

static void screem_pref_dialog_toggle_notify( GConfClient *client,
					      guint cnxn_id,
					      GConfEntry *entry,
					      gpointer data )
{
	if( entry->value && entry->value->type == GCONF_VALUE_BOOL ) {
		GtkWidget *toggle;
		gboolean wstate;
		gboolean state;
		
		toggle = GTK_WIDGET( data );

		wstate = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(toggle));
		state = gconf_value_get_bool( entry->value );

		if( wstate != state ) {
			gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(toggle),
						      state );
		}
	}
}

static void screem_pref_dialog_spin_notify( GConfClient *client,
					    guint cnxn_id,
					    GConfEntry *entry,
					    gpointer data )
{
	if( entry->value && entry->value->type == GCONF_VALUE_INT ) {
		GtkWidget *spin;
		gint value;
		gint nvalue;
		
		spin = GTK_WIDGET( data );
		value = gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( spin ) );
		nvalue = gconf_value_get_int( entry->value );

		if( value != nvalue ) {
			gtk_spin_button_set_value( GTK_SPIN_BUTTON( spin ),
						   nvalue );
		}
	}
}

static void screem_pref_dialog_colour_notify( GConfClient *client,
					      guint cnxn_id,
					      GConfEntry *entry,
					      gpointer data )
{
	if( entry->value && entry->value->type == GCONF_VALUE_STRING ) {
		const gchar *value; 
		GdkColor colour;
		GtkWidget *widget;
		GdkColor pcolour;
		gushort junk;
	
		value = gconf_value_get_string( entry->value );
		widget = GTK_WIDGET( data );
		gnome_color_picker_get_i16( GNOME_COLOR_PICKER( widget ),
				    &pcolour.red, &pcolour.green, 
				    &pcolour.blue, &junk );

		if( gdk_color_parse( value, &colour ) && 
		    ! gdk_color_equal( &colour, &pcolour ) ) {
			gnome_color_picker_set_i16( GNOME_COLOR_PICKER(widget),
						    colour.red, colour.green,
						    colour.blue, 0 );
		}
	}
}

static void screem_pref_dialog_radio_notify( GConfClient *client,
					    guint cnxn_id,
					    GConfEntry *entry,
					    gpointer data )

{
	if( entry->value && entry->value->type == GCONF_VALUE_INT ) {
		GtkWidget *toggle;
		const gchar *name;
		gboolean wstate;
		gint gval;
		gint val;
		
		toggle = GTK_WIDGET( data );
		name = gtk_widget_get_name( toggle );

		wstate= gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(toggle));
		gval = gconf_value_get_int( entry->value );
		
		val = GPOINTER_TO_INT( g_object_get_data( G_OBJECT( toggle ),
							  "val" ) );

		if( val != gval && wstate ) {
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(toggle),
						     FALSE );
		} else if( val == gval && ! wstate ) {
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(toggle),
						     TRUE );
		}
	}
}

static void screem_pref_dialog_string_notify( GConfClient *client,
					      guint cnxn_id,
					      GConfEntry *entry,
					      gpointer data )
{
	if( entry->value && entry->value->type == GCONF_VALUE_STRING ) {
		const gchar *value;
		GtkWidget *widget;
		GtkWidget *gtkentry;

		value = gconf_value_get_string( entry->value );
		widget = GTK_WIDGET( data );
		gtkentry = NULL;
		
		if( GNOME_IS_ENTRY( widget ) ) {
			gtkentry = gnome_entry_gtk_entry( GNOME_ENTRY(widget));
		} else if( GTK_IS_ENTRY( widget ) ) {
			gtkentry = widget;
		} else if( GTK_IS_COMBO( widget ) ) {
			gtkentry = GTK_COMBO( widget )->entry;
		} else if( GNOME_IS_FILE_ENTRY( widget ) ) {
			gtkentry = gnome_file_entry_gtk_entry( GNOME_FILE_ENTRY( widget ) );
		} else if( GNOME_IS_ICON_ENTRY( widget ) ) {
			const gchar *cvalue;
			cvalue = gnome_icon_entry_get_filename( GNOME_ICON_ENTRY( widget ) );
			if( ! cvalue || strcmp( cvalue, value ) )
				gnome_icon_entry_set_filename( GNOME_ICON_ENTRY( widget ),
							      value);
		} else if( GNOME_IS_FONT_PICKER( widget ) ) {
			const gchar *cvalue;
			cvalue = gnome_font_picker_get_font_name( GNOME_FONT_PICKER( widget ) );
			if( ! cvalue || strcmp( cvalue, value ) )
				gnome_font_picker_set_font_name( GNOME_FONT_PICKER( widget ), value );
			
		}
		if( gtkentry ) {
			const gchar *cvalue;
			cvalue = gtk_entry_get_text( GTK_ENTRY( gtkentry ) );
			if( value && strcmp( value, cvalue ) ) {
				gtk_entry_set_text( GTK_ENTRY( gtkentry ),
						    value );
			}
		}
	}
}

static gboolean screem_pref_dialog_toggle_changed( GtkWidget *toggle )
{
	GConfClient *client;
	gboolean state;
	const gchar *key;

	client = gconf_client_get_default();

	state = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( toggle ) );
	
	key = g_object_get_data( G_OBJECT( toggle ), "key" );
	gconf_client_set_bool( client, key, state, NULL );

	g_object_unref( client );
	
	return TRUE;
}


static void screem_pref_dialog_spin_changed( GtkWidget *spin )
{
	GConfClient *client;
	const gchar *key;
	gdouble value;

	client = gconf_client_get_default();
	key = g_object_get_data( G_OBJECT( spin ), "key" );
	
	value = gtk_spin_button_get_value( GTK_SPIN_BUTTON( spin ) );

	gconf_client_set_float( client, key, value, NULL );

	g_object_unref( client );
}


static gboolean screem_pref_dialog_icon_changed( GtkWidget *widget )
{
	GConfClient *client;
	const gchar *key;
	gchar *value;

	client = gconf_client_get_default();
	key = g_object_get_data( G_OBJECT( widget ), "key" );
	
	value = gnome_icon_entry_get_filename( GNOME_ICON_ENTRY( widget ) );
	if( ! value )
		value = g_strdup( "" );

	gconf_client_set_string( client, key, value, NULL );

	g_free( value );

	g_object_unref( client );
	
	return TRUE;
}

static gboolean screem_pref_dialog_string_set( GtkWidget *widget )
{
	GtkWidget *entry = NULL;
	const gchar *cvalue = NULL;
	const gchar *key;

	GConfClient *client;

	client = gconf_client_get_default();
	
	key = g_object_get_data( G_OBJECT( widget ), "key" );

	if( GNOME_IS_ENTRY( widget ) ) {
		entry = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
		cvalue = gtk_entry_get_text( GTK_ENTRY( entry ) );
	} else if( GTK_IS_COMBO( widget ) ) {
		entry = GTK_COMBO( widget )->entry;
		cvalue = gtk_entry_get_text( GTK_ENTRY( entry ) );
	} else if( GNOME_IS_FILE_ENTRY( widget ) ) {
		entry = gnome_file_entry_gtk_entry( GNOME_FILE_ENTRY( widget ) );
		cvalue = gtk_entry_get_text( GTK_ENTRY( entry ) );	
	} else if( GTK_IS_ENTRY( widget ) ) {
		cvalue = gtk_entry_get_text( GTK_ENTRY( widget ) );
	}
	    
	if( cvalue ) {
		gconf_client_set_string( client, key, 
					 cvalue, NULL );
	}

	g_object_unref( client );
	
	return TRUE;
}


static void screem_pref_dialog_font_set( GtkWidget *fp,
					  const gchar *font )
{
	GConfClient *client;
	const gchar *key;

	client = gconf_client_get_default();
	key = g_object_get_data( G_OBJECT( fp ), "key" );
	gconf_client_set_string( client, key, font, NULL );
	g_object_unref( client );
}

static void screem_pref_dialog_colour_set( GnomeColorPicker *cp,
					   guint r, guint g, guint b,
					   guint a )
{
	GConfClient *client;
	const gchar *key;
	gchar *value;

	client = gconf_client_get_default();
	key = g_object_get_data( G_OBJECT( cp ), "key" );
	value = g_strdup_printf( "#%.4x%.4x%.4x", r, g, b );
	gconf_client_set_string( client, key, value, NULL );
	g_free( value );
	g_object_unref( client );
}

static gboolean screem_pref_dialog_radio_changed( GtkWidget *toggle )
{
	GConfClient *client;
	gboolean state;
	gint val;
	const gchar *key;

	client = gconf_client_get_default();
	val = GPOINTER_TO_INT( g_object_get_data( G_OBJECT( toggle ),
						  "val" ) );
	state = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( toggle ) );
	
	key = g_object_get_data( G_OBJECT( toggle ), "key" );
	
	if( state ) {
		gconf_client_set_int( client, key, val, NULL );
	}
	
	g_object_unref( client );
	
	return TRUE;
}


/* public stuff */

ScreemPrefDialog *screem_pref_dialog_new()
{
	ScreemPrefDialog *dialog;

	dialog = g_object_new( SCREEM_TYPE_PREF_DIALOG, 
				"title", _( "Preferences - Screem" ),
				NULL );

	return dialog;
}

ScreemPrefDialog *screem_pref_dialog_new_with_logo( GdkPixbuf *pixbuf )
{
	ScreemPrefDialog *dialog;

	dialog = g_object_new( SCREEM_TYPE_PREF_DIALOG, 
				"title", _( "Preferences - Screem" ),
				"logo", pixbuf,
				NULL );


	return dialog;
}

void screem_pref_dialog_add_page( ScreemPrefDialog *dialog,
				const gchar *category,
				const gchar *name,
				GtkWidget *child )
{
	screem_pref_dialog_add_page_with_stock( dialog, category, name,
						NULL, child );
}

void screem_pref_dialog_add_page_with_stock( ScreemPrefDialog *dialog,
					const gchar *category,
					const gchar *name,
					const gchar *stock_id,
					GtkWidget *child )
{
	GtkTreeIter it;
	GtkTreeIter node;
	gboolean add;
	GtkTreeModel *categories;
	GtkNotebook *notebook;
	GtkWidget *pref_page;
	GtkWidget *label;
	gint page;

	
	g_return_if_fail( SCREEM_IS_PREF_DIALOG( dialog ) );
	g_return_if_fail( category != NULL );
	g_return_if_fail( name != NULL );
	g_return_if_fail( child != NULL );
	
	categories = GTK_TREE_MODEL( dialog->priv->categories );

	add = TRUE;

	if( gtk_tree_model_get_iter_first( categories, &it ) ) {
		
		if( ! screem_pref_dialog_find_category( dialog, &it, category ) ) {
			screem_pref_dialog_add_category( dialog, category, NULL, NULL, &it, -1 );
		}
		
		add = !( gtk_tree_model_iter_children( categories, &node, &it ) &&
		    	 screem_pref_dialog_find_category( dialog, &node, name ) );
		if( ! add ) {
			g_warning( "Duplicate page %s/%s\n", category, name );
		}
	} else {
		screem_pref_dialog_add_category( dialog, category, NULL, NULL, &it, -1 );
	}
	if( add ) {
		notebook = dialog->priv->notebook;
		pref_page = screem_pref_dialog_new_page( dialog, name, stock_id, child );
		label = gtk_label_new( name );
		gtk_notebook_append_page( notebook, pref_page, label );
		page = gtk_notebook_get_n_pages( notebook ) - 1;
		screem_pref_dialog_add_category( dialog, name, stock_id, &it, &node, page );
	}
	gtk_tree_view_expand_all( GTK_TREE_VIEW( dialog->priv->view ) );
}

void screem_pref_dialog_add_pref( ScreemPrefDialog *dialog,
				const gchar *key,
				ScreemPrefNotifyType type,
				gint value,
				GtkWidget *widget )
{
	ScreemPrefDialogPrivate *priv;
	GConfClientNotifyFunc func;
	guint handle;

	gboolean tval;
	gint ival;
	gdouble spval;
	gchar *sval;
	GdkColor colour;
	GtkWidget *gtkentry;
	
	g_return_if_fail( SCREEM_IS_PREF_DIALOG( dialog ) );
	g_return_if_fail( key != NULL );
	g_return_if_fail( widget != NULL );
	
	priv = dialog->priv;
	
	g_object_set_data( G_OBJECT( widget ), "key", (gchar*)key );
	
	switch( type ) {
		case SCREEM_PREF_TOGGLE:
			func = screem_pref_dialog_toggle_notify;
			tval = gconf_client_get_bool( priv->client, key, NULL );
			g_signal_connect( G_OBJECT( widget ), "toggled",
					  G_CALLBACK( screem_pref_dialog_toggle_changed ),
					  NULL );
			gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( widget ),
							tval );
			break;
		case SCREEM_PREF_RADIO:
			func = screem_pref_dialog_radio_notify;
			g_object_set_data( G_OBJECT( widget ), "val", 
				   	   GINT_TO_POINTER( value ) );
			ival = gconf_client_get_int( priv->client, key, NULL );
			if( value == ival ) {
				gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( widget ),
								TRUE );
			}
			g_signal_connect( G_OBJECT( widget ), "toggled",
					  G_CALLBACK( screem_pref_dialog_radio_changed ),
					  NULL );
			break;
		case SCREEM_PREF_SPIN:
			func = screem_pref_dialog_spin_notify;
			spval = gconf_client_get_float( priv->client, key, NULL );
			g_signal_connect( G_OBJECT( widget ), "value_changed",
					  G_CALLBACK( screem_pref_dialog_spin_changed ),
					  NULL );
			gtk_spin_button_set_value( GTK_SPIN_BUTTON( widget ),
						   spval );
			break;
		case SCREEM_PREF_COLOUR:
			func = screem_pref_dialog_colour_notify;
			sval = gconf_client_get_string( priv->client, key, NULL );
			g_signal_connect( G_OBJECT( widget ), "color_set",
					  G_CALLBACK( screem_pref_dialog_colour_set ),
					  NULL );
			if( gdk_color_parse( sval, &colour ) ) {
				gnome_color_picker_set_i16( GNOME_COLOR_PICKER(widget),
							    colour.red, colour.green,
							    colour.blue, 0 );
			}
			break;
		case SCREEM_PREF_STRING:
			func = screem_pref_dialog_string_notify;
			sval = gconf_client_get_string( priv->client, key, NULL );
			gtkentry = NULL;
			if( GNOME_IS_ENTRY( widget ) ) {
				g_signal_connect( G_OBJECT( widget ), "changed",
						  G_CALLBACK( screem_pref_dialog_string_set ),
						  NULL );
				gtkentry = gnome_entry_gtk_entry( GNOME_ENTRY(widget));
			} else if( GTK_IS_ENTRY( widget ) ) {
				g_signal_connect( G_OBJECT( widget ), "changed",
						  G_CALLBACK( screem_pref_dialog_string_set ),
						  NULL );
				gtkentry = widget;
			} else if( GTK_IS_COMBO( widget ) ) {
				gtkentry = GTK_COMBO( widget )->entry;
				g_object_set_data( G_OBJECT( gtkentry ), "key", (gchar*)key );
				g_signal_connect( G_OBJECT( gtkentry ), "changed",
						  G_CALLBACK( screem_pref_dialog_string_set ),
						  NULL );
			} else if( GNOME_IS_FILE_ENTRY( widget ) ) {
				g_signal_connect( G_OBJECT( widget ), "changed",
						  G_CALLBACK( screem_pref_dialog_string_set ),
						  NULL );
				gtkentry = gnome_file_entry_gtk_entry( GNOME_FILE_ENTRY( widget ) );
			} else if( GNOME_IS_ICON_ENTRY( widget ) ) {
				g_signal_connect( G_OBJECT( widget ), "changed",
						  G_CALLBACK( screem_pref_dialog_icon_changed ),
						  NULL );
				gnome_icon_entry_set_filename( GNOME_ICON_ENTRY( widget ),
							       sval);
			} else if( GNOME_IS_FONT_PICKER( widget ) ) {
				g_signal_connect( G_OBJECT( widget ), "font_set",
						  G_CALLBACK( screem_pref_dialog_font_set ),
						  NULL );
				gnome_font_picker_set_font_name( GNOME_FONT_PICKER( widget ), sval );
			}
			if( gtkentry && sval ) {
				gtk_entry_set_text( GTK_ENTRY( gtkentry ), sval );
			}
			g_free( sval );
			break;
		default:
			g_warning( "Invalid preference notify type\n" );
			return;
			break;
	}

	handle = gconf_client_notify_add( priv->client,
					  key, func,
					  widget, NULL, NULL );

	priv->notify_handles = g_slist_prepend( priv->notify_handles,
						GUINT_TO_POINTER( handle ) );
}
